;**************************************************************************
;    Filename:	    swdelic.asm                                           *
;    Author:        Jaroslav Belza                                        *
;                                                                         *
; SW delic M/N - M inpuls na vstupu generuje N impuls na vstupu        *
;**************************************************************************
; 
;                  ----------------------  
;            +5V --|Vdd              Vss|-- GND
;    nen pouit --|GP5/oscin    GP0/PGD|-- nic (ICSP data)
; vstup impulz --|GP4/oscout   GP1/PGC|-- nic (ICSP clk)
; nic (ICSP Vpp) --|GP3/MCLR/Vpp GP2/INT|-- vstup impulz (T0CKI)
;                  ----------------------
;**********************************************************************

	list      p=12f629           ; list directive to define processor
	#include <p12f629.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT


;***** VARIABLE DEFINITIONS
;w_temp        EQU     0x20        ; variable used for context saving 
;status_temp   EQU     0x21        ; variable used for context saving
count		equ	0x30	; poitadlo peruen
suma_hi		equ	0x31	; souet pro vpoet vstupnho impulsu
suma_lo		equ	0x32	; souet pro vpoet vstupnho impulsu
div_hi		equ	0x33	; slo pro dlen a zbytek po dlen
div_lo		equ	0x34	; slo pro dlen a vsledek

	#define	OUTP	GPIO,4		; vstup impulz
	
;**********************************************************************
		ORG     0x000             ; processor reset vector
	bcf     STATUS,RP0		; set bank 0
	bsf 	OUTP			; nastav vstup do H
	goto    main              ; go to beginning of program

		ORG     0x004             ; interrupt vector location

;nen teba ukldat, v hlavn smyce se nic nedje
;	movwf   w_temp            ; save off current W register contents
;	movf	STATUS,w          ; move status register into W register
;	movwf	status_temp       ; save off contents of STATUS register

	btfss	INTCON,INTF		; peruen od INT?
	goto	endint			; ne, tak ven

	call	0x3f1			; zvy obsah registr suma 2x o N
	addwf	suma_lo,f
	btfsc	STATUS,C
	incf	suma_hi,f
	addwf	suma_lo,f
	btfsc	STATUS,C
	incf	suma_hi,f
	movf	suma_lo,w		; pesun sla na dlen, suma mus bt zachovna
	movwf	div_lo
	movf	suma_hi,w
	movwf	div_hi
	call	0x3f0		; dlitel (M)

;dlen suma slem M
	call	divide
; chyby se po nvratu netestuj

	btfss	div_lo,0	; testuj nejni bit vsledku
	bsf 	OUTP		; zhasni LED
	btfsc	div_lo,0	; testuj nejni bit vsledku
	bcf 	OUTP		; rozsvi LED

	decfsz	count,f
	goto	endint
	call	0x3f0		; jsme na konci cyklu, jedeme znovu
	movwf	count		; poten nastaven poitadla peruen (M)
	clrf	suma_hi		; vynuluj souin potu impulz a puls2
	clrf	suma_lo

endint
	bcf		INTCON,INTF		; vynuluj pznak peteen INT
; nen teba obnovovat
;	movf    status_temp,w     ; retrieve copy of STATUS register
;	movwf	STATUS            ; restore pre-isr STATUS register contents
;	swapf   w_temp,f
;	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt

main
; inicializace osciltoru
	call    0x3FF			; retrieve factory calibration value
	bsf     STATUS,RP0		; set file register bank to 1 
	movwf   OSCCAL			; (b1)update register with factory cal value

; nastaven port
	movlw	0x38			; pull-up ano, T0CKI, peddlika 1:1 pro WDT
	movwf	OPTION_REG		; (b1)
	movlw	0x2f			; GP4 je vstup, ostatn vstupy b'00101111'
	movwf	TRISIO			; (b1)
	movlw	0x23			; 27h pokud nen pull-up na vstupu GP2 b'00100011'
	movwf	WPU 			; (b1) nastaven WPU - nen teba pokud jsou pull-up na vech vstupech
	bcf     STATUS,RP0		; set bank 0

; nastaven peruen
	movlw	0x90			; (b0b1) povoleno peruen od GP2/INT
	movwf	INTCON

	bsf 	OUTP			; zhasni LED
	call	0x3f0			; poten nastaven poitadla peruen (M)
	movwf	count
	clrf	suma_hi			;vynuluj souin potu impuls a N
	clrf	suma_lo

; hlavn smyka
loop
	clrwdt
	goto	loop			; vechno se dje bhem peruen


; Podprogramy -------------------------------
;	org	100h

divide
; Dlen 16/8
; http://www.piclist.com/techref/microchip/math/div/16by8lz.htm
; Call w/: Number in div_hi:div_lo, divisor in W.
; Returns: Quotient in div_lo, remainder in div_hi. W preserved.
;          Carry set if error. Z if divide by zero, NZ if divide overflow.
; Notes:   Works by left shifted subtraction.
;          Size = 29, Speed(w/ call&ret) = 7 cycles if div by zero
;          Speed = 94 minimum, 129 maximum cycles

	addlw	0			; w+=0 (to test for div by zero)
	bsf 	STATUS,C	; set carry in case of error
	btfsc	STATUS,Z    ; if zero
	return				; return (error C,Z)

	call	DivSkipHiShift

	call	DivCode
	call	DivCode
	call	DivCode
	call	DivCode
	call	DivCode
	call	DivCode
	call	DivCode
	call	DivCode

	rlf 	div_lo,f	; C << lo << C

    ; If the first subtract didn't underflow, and the carry was shifted
    ; into the quotient, then it will be shifted back off the end by this
    ; last RLF. This will automatically raise carry to indicate an error.
    ; The divide will be accurate to quotients of 9-bits, but past that
    ; the quotient and remainder will be bogus and carry will be set.

	bcf	STATUS,Z		; NZ (in case of overflow error)
	return				; we are done!

DivCode
	rlf 	div_lo,f	; C << lo << C
	rlf 	div_hi,f	; C << hi << C
	btfss	STATUS,C		; if Carry
	goto	DivSkipHiShift	;
	subwf	div_hi,f		;   hi-=w
	bsf 	STATUS,C		;   ignore carry
	return					;   done
							; endif
DivSkipHiShift
	subwf	div_hi,f	; hi-=w
	btfsc	STATUS,C	; if carry set
	return				;   done
	addwf	div_hi,f	; hi+=w
	bcf		STATUS,C	; clear carry
	return				; done

;---------------

	org 3f0h
	retlw	.10		; poet vstupnch impuls - M (M > 2*N)
	retlw	.3		; poet vstupnch impuls - N


; initialize eeprom locations

		ORG	0x2100
		DE	"SW delic m/n - v2. Jaroslav Belza 2013 - jbelza@centrum.cz"


	END                       ; directive 'end of program'

